///# Skiplist
///<p>skiplist is an important data structure of database and many database use this as the basic structure in memory.
/// In the database I also use two Skiplist in memory to store datas temperary.</p>
///</p> We refer the source of leveldb,however we know that the first time, google use C++ writting leveldb and then rewrite it by go.</p> 

#[allow(unused)]
pub mod skip_lists
{
    static mut  COUNTS:i32=0;
    use std::{borrow::{Borrow, BorrowMut}, cell::{Ref, RefCell}, collections::btree_map::Values, fmt::Display, rc::Rc,time::{Duration, Instant},cmp::PartialOrd};
    use crate::algorithm::{baisc_data::basic_data, random};
    use crate::algorithm::baisc_data;
    use rand::Rng;
    
    #[derive(PartialEq)]
    pub enum InsertResult
    {
        Successful,
        Fault,
        Exist
    }
    #[derive(Debug,Clone)]
    ///## Node of skiplist
    ///
    /// <p>We will build a kv database,so the node has two members key and value.</p>
    ///
    /// <p>The node would never be used out of the mod.</p>
    pub struct Node<K:PartialOrd,V>
    where  K:Copy+Display,
           V:Clone+Display                             
    {
        key:K,
        value:V,
        level:usize,
        forward:Vec<Option<Rc<RefCell<Node<K,V>>>>>
    }
    impl<K:PartialOrd,V> Node<K,V>
    where  K:Copy+Display, 
           V:Clone+Display{
        fn new(_key:K,_value:V,_level:usize)->Rc<RefCell<Self>>
        {
            Rc::new(
                RefCell::new(
                    Node
                    {
                        key:_key,
                        value:_value,
                        level:_level,
                        forward:vec![None;_level+1]
                    }
                )
            )
        }
    }
     
    ///## Skiplist
    /// 
    /// 
    #[derive(Debug)]
    pub struct Skiplists<K,V>
    where  K:Copy+Display,
           K:PartialOrd,
           V:Clone+Display 
    {
        max_level:usize,
        current_level:usize,
        current_size:usize,
        max_size:usize,
        head:Vec<Option<Rc<RefCell<Node<K,V>>>>>
    }

    impl<K:PartialOrd,V> Skiplists<K,V>
    where  K:Copy+Display,
           V:Clone+Display 
    {
        pub fn new(_max_size:usize)->Self
        {
            
            Skiplists
            {
                max_level:_max_size,
                current_level:0,
                current_size:0,
                max_size:basic_data::MAX_SIZE_OF_SKIPLIST,
                head:vec![None;_max_size+1]

            }
        }

        fn creat_node(_key:K,_value:V,_level:usize)->Rc<RefCell<Node<K,V>>>
        {
            Node::new(_key, _value, _level)
        }

        
        pub fn insert(&mut self,key:K,value:V)->InsertResult
        {
            let mut rng: rand::prelude::ThreadRng = rand::thread_rng();
            if self.current_size==0                //如果shiplist是全空的,直接将点插入
            {
                //let random_level: usize=random::random::lgc(self.max_level);  没空研究随机数算法，先用一下别人的
                let random_level: usize=rng.gen_range(1..(self.max_level+1)); 
                let new_node: Rc<RefCell<Node<K, V>>>=Self::creat_node(key,value,random_level);      //创建新的节点
                for i in 0..=(random_level-1)
                {
                    self.head[i]=Some(new_node.clone());
                }
                self.current_size=self.current_size+1;
                self.current_level=random_level;
            }
            else                    //当list不为空的时候需要判定插入的位置
            {
                let random_level:usize=rng.gen_range(1..(self.max_level+1));
                let mut cur_point:Option<Rc<RefCell<Node<K,V>>>>=None;
                let mut cur_node:Vec<Option<Rc<RefCell<Node<K,V>>>>>=vec![None;self.max_level];
                let mut cur_level: i32=(self.current_level as i32)-1;
                while(cur_level>=0)
                {
                    if self.head[cur_level as usize].is_none()                //如果起始点为空则直接跳过（在这一层不会有节点）
                    {
                        cur_level=cur_level-1;
                        continue;
                    }
                    else 
                    {                                        //判定每一层的起始位置
                        if cur_level >=((self.max_level) as i32)-1 ||  cur_node[(cur_level as usize+1)].is_none()         
                        {

                            cur_point=self.head[cur_level as usize].clone();
                            if (*(cur_point.clone().unwrap())).borrow().key>key                  //如果第一个节点key就大于当前数据（主要是没有一个空的头节点就比较复杂）
                            {
                                cur_node[cur_level as usize]=None;
                                cur_level=cur_level-1;
                                continue;          
                            }
                            else if (*(cur_point.clone().unwrap())).borrow().key==key
                            {
                                (*(*(cur_point.clone().unwrap())).borrow_mut()).value=value.clone();
                                return InsertResult::Exist;
                            } 
                            
                        }
                        loop 
                        {
                            if (*(*(cur_point.clone().unwrap())).borrow_mut()).forward[cur_level as usize].is_none()
                            {
                                cur_node[cur_level as usize]=cur_point.clone();
                                break;
                            }
                            else if (*(((*(*(cur_point.clone().unwrap())).borrow_mut()).forward[cur_level as usize]).clone()).unwrap()).borrow().key==key
                            {
                                (*(((*(*(cur_point.clone().unwrap())).borrow_mut()).forward[cur_level as usize]).clone()).unwrap()).borrow_mut().value=value;      //将对应的节点数据修改
                                return InsertResult::Exist;
                            }
                            else if (*(((*(*(cur_point.clone().unwrap())).borrow_mut()).forward[cur_level as usize]).clone()).unwrap()).borrow().key>key
                            {
                                cur_node[cur_level as usize]=cur_point.clone();
                                break;
                            }
                            else
                            {
                                cur_point=(*((cur_point.clone()).unwrap())).borrow().forward[cur_level as usize].clone();
                            }
                        }
                        cur_level=cur_level-1;
                    }
                }
                let mut middle_ptr:Option<Rc<RefCell<Node<K,V>>>>=None;
                let mut new_nodes:Rc<RefCell<Node<K, V>>> =Self::creat_node(key, value, random_level);
                if random_level>self.current_level
                {
                    self.current_level=random_level;
                }
                
                for i in 0..=(random_level-1)
                {
                    if cur_node[i].is_none()             //前面给出了两种none的情况一种是这一层什么都没有，一种是这一层第一个节点就大于当前
                    {
                        if self.head[i].is_none()
                        {
                            self.head[i]=Some(new_nodes.clone());
                        }
                        else {
                            (*new_nodes).borrow_mut().forward[i]=self.head[i].clone();
                            self.head[i]=Some(new_nodes.clone());
                        }
                    }
                    else
                    {
                        (*new_nodes).borrow_mut().forward[i]=((*(*((cur_node[i].clone()).unwrap())).borrow_mut()).forward[i]).clone();
                        (*(cur_node[i].clone().unwrap())).borrow_mut().forward[i]=Some(new_nodes.clone());
                    }
                }
                self.current_size=self.current_size+1;
            }
            InsertResult::Successful   
        }

        pub fn search(&self,key:K) ->Option<Rc<RefCell<Node<K,V>>>>              //查找数据返回对应节点指针
        {
            let mut middle:Option<Rc<RefCell<Node<K, V>>>>=self.head[(self.current_level-1)].clone();
            let mut tmp_level: i32=(self.current_level-1) as i32;
            loop
            {
                if (*((*((middle.clone()).unwrap())).borrow())).key==key              //头节点等于所需查找的节点
                {
                    return middle.clone();
                }
                else if (*((*((middle.clone()).unwrap())).borrow())).key<key
                {
                    break;
                } 
                else {
                    tmp_level=tmp_level-1;
                }
                //当查找到最下面的时候所有的key都大于需要的说明不存在
                if(tmp_level>=0)                
                {
                    middle=self.head[tmp_level as usize].clone();
                }
                else {
                    return None;
                }
            }
            while(tmp_level>=0)                
            {
                while(!((*((middle.clone()).unwrap())).borrow().forward[tmp_level as usize]).is_none())    //如果当这个下一个节点非空的时候非空的时候
                {
                    if ((*((*((middle.clone()).unwrap())).borrow().forward[tmp_level as usize]).clone().unwrap()).borrow().key)>=key
                    {
                        if ((*((*((middle.clone()).unwrap())).borrow().forward[tmp_level as usize]).clone().unwrap()).borrow().key)==key
                        {
                            return ((*((middle.clone()).unwrap())).borrow().forward[tmp_level as usize]).clone();
                        }
                        else {
                            break;
                        }
                    }
                    else {  
                        middle=(*((middle.clone()).unwrap())).borrow().forward[tmp_level as usize].clone();             //向后推进到下一个节点
                    }
                }
                tmp_level=tmp_level-1;
            }
            return None     //如果查找失败返回一个None
        }




        pub fn printskip(&self,level:usize)
        {

            let mut middle:Option<Rc<RefCell<Node<K, V>>>>=self.head[level].clone();            //直接遍历第0行即可
            while !((middle.clone()).is_none())
            {
                print!("(");
                print!("key={}",((*(middle.clone().unwrap())).borrow()).key);
                print!("value={}",((*(middle.clone().unwrap())).borrow()).value);
                print!(")=>");
                middle=(*((middle.clone()).unwrap())).borrow().forward[level].clone();

                unsafe
                {
                    COUNTS=COUNTS+1;
                    println!("COUNT={}",COUNTS);
                }
            }
        }
    }
}
